<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/time_tracker/credits.htm
// +----------------------------------------------------------------------+

import('ttTimeHelper');
import('ttTeamHelper');

$mdb2 = getConnection();

$report_days = $period->getAllDays();
// Prepare $dates and $weekdays arrays.
// They are used in report_timesheet_old.php, report_timesheet_csv_old.php, and report_timesheet_xml_old.php for output.
foreach ($report_days as $single_day) {
  $index = $single_day->toString(DB_DATEFORMAT);
  $weekdays[$index] = $single_day->getDay(); // Array of weekday numbers for all dates in report.
  $dates[$index] = $single_day->toString();  // Array of formatted dates for all dates in report.
}

$totals_only = $bean->getAttribute('chtotalsonly');

$fields = array(); // An array of fields for database query.
$header = array(); // An array of labels for report header.
$hkeys  = array(); // An array of keys for header labels.

// ID is needed for sorting records.
array_push($fields, 'l.id as id');

// Date is always required.
array_push($fields, 'l.date as date');
array_push($header, $i18n->getKey('label.date'));
array_push($hkeys, 'date');

// User name - needed when a  manager / co-managers view reports.
if($user->canManageTeam()) {
  array_push($fields, 'u.name as user');
  array_push($header, $i18n->getKey('label.user'));
  array_push($hkeys, 'user');
}

// Add client name if it is selected.
if ($bean->getAttribute('chclient')) {
  array_push($fields, 'c.name as client');
  array_push($header, $i18n->getKey('label.client'));
  array_push($hkeys, 'client');
}

// Add project name if it is selected or we are grouping by project.
if ($bean->getAttribute('chproject') || $bean->getAttribute('sortby')=='project') {
  array_push($fields, 'p.name as project');
  array_push($header, $i18n->getKey('label.project'));
  array_push($hkeys, 'project');
}

// Add task name if it is selected or we are grouping by task.
if ($bean->getAttribute('chtask') || $bean->getAttribute('sortby')=='task') {
  array_push($fields, 't.name as task');
  array_push($header, $i18n->getKey('label.task'));
  array_push($hkeys, 'task');
}

// Handle custom field (in header and key arrays). Because we need to join tables in this case, sql needs to be modified
// by a more complicated method rather than using the fields array.
$include_cf_1 = $bean->getAttribute('chcf_1');
if ($include_cf_1) {
  require_once('../plugins/CustomFields.class.php');
  $custom_fields = new CustomFields($user->team_id);
  $cf_1_type = $custom_fields->fields[0]['type'];
  if ($cf_1_type == CustomFields::TYPE_TEXT) {
    array_push($fields, 'cfl.value as cf_1');	
  } else if ($cf_1_type == CustomFields::TYPE_DROPDOWN) {
  	array_push($fields, 'cfo.value as cf_1');	
  }
  array_push($header, $custom_fields->fields[0]['label']);
  array_push($hkeys, 'cf_1');
}

// Add start time.
if ($bean->getAttribute('chstart')) {
  array_push($fields, "TIME_FORMAT(l.start, '%k:%i') as start");
  array_push($header, $i18n->getKey('label.start'));
  array_push($hkeys, 'start');
}

// Add finish time.
if ($bean->getAttribute('chfinish')) {
  array_push($fields, "TIME_FORMAT(sec_to_time(time_to_sec(l.start) + time_to_sec(l.duration)), '%k:%i') as finish");
  array_push($header, $i18n->getKey('label.finish'));
  array_push($hkeys, 'finish');
}

// Add duration.
if ($bean->getAttribute('chduration')) {
  array_push($fields, "TIME_FORMAT(l.duration, '%k:%i') as duration");
  array_push($header, $i18n->getKey('label.duration'));
  array_push($hkeys, 'duration');
}

// Add note.
if ($bean->getAttribute('chnote')) {
  array_push($fields, 'l.comment as note');
  array_push($header, $i18n->getKey('label.note'));
  array_push($hkeys, 'note');
}

// Handle cost.
$includeCost = $bean->getAttribute('chcost');
if ($includeCost) {
  if (MODE_TIME == $user->tracking_mode)
    array_push($fields, "format(l.billable * coalesce(u.rate, 0) * time_to_sec(l.duration)/3600, 2) as cost");   // Use default user rate.
  else
    array_push($fields, "format(l.billable * coalesce(upb.rate, 0) * time_to_sec(l.duration)/3600, 2) as cost"); // Use project rate for user.
  array_push($header, $i18n->getKey('label.cost'));
  array_push($hkeys, 'cost');
}

// Add invoice name if it is selected.
if ($user->canManageTeam() && $bean->getAttribute('chinvoice')) {
  array_push($fields, 'i.name as invoice');
  array_push($header, $i18n->getKey('label.invoice'));
  array_push($hkeys, 'invoice');
}

// Prepare sql query parts for client, project, task, and billable dropdowns.
$dropdown_parts = '';
if ($bean->getAttribute('client')) $dropdown_parts .= ' and l.client_id = '.$bean->getAttribute('client');
if ($bean->getAttribute('project')) $dropdown_parts .= ' and l.project_id = '.$bean->getAttribute('project');
if ($bean->getAttribute('task')) $dropdown_parts .= ' and l.task_id = '.$bean->getAttribute('task');
if ($bean->getAttribute('include_records')=='1') $dropdown_parts .= ' and l.billable = 1';
if ($bean->getAttribute('include_records')=='2') $dropdown_parts .= ' and l.billable = 0';
  
// Sorting key.
$sort_key = $bean->getAttribute('sortby');

// Determine names of currently selected project and activity.
$current_project_name = '';
$current_task_name = '';

if ($bean->getAttribute('chproject') && $bean->getAttribute('project')) {
  $sql = 'select name from tt_projects where id = '.$bean->getAttribute('project');
  $res = $mdb2->query($sql);

  if (!is_a($res, 'PEAR_Error')) {
  	if ($val = $res->fetchRow()) {
      $current_project_name = $val['name'];      
    }
  }
}

if ($bean->getAttribute('chtask') && $bean->getAttribute('task')) {
  $sql = 'select name from tt_tasks where id = '.$bean->getAttribute('task');
  $res = $mdb2->query($sql);
  
  if (!is_a($res, 'PEAR_Error')) {
  	if ($val = $res->fetchRow()) {
      $current_task_name = $val['name'];
    }
  }
}

if(!isset($userlist)) $userlist = -1;

// Prepare sql query part for left joins.
$left_joins = null;
if ($bean->getAttribute('chclient'))
  $left_joins .= " left join tt_clients c on (c.id = l.client_id)";
if ($user->canManageTeam() && $bean->getAttribute('chinvoice'))
  $left_joins .= " left join tt_invoices i on (i.id = l.invoice_id)";
if ($user->canManageTeam())
   $left_joins .= " left join tt_users u on (u.id = l.user_id)";
if ($bean->getAttribute('chproject') || $bean->getAttribute('sortby')=='project')
  $left_joins .= " left join tt_projects p on (p.id = l.project_id)";	
if ($bean->getAttribute('chtask') || $bean->getAttribute('sortby')=='task')
  $left_joins .= " left join tt_tasks t on (t.id = l.task_id)";		
if ($include_cf_1) {
  if ($cf_1_type == CustomFields::TYPE_TEXT)
    $left_joins .= " left join tt_custom_field_log cfl on (l.id = cfl.log_id)";
  else if ($cf_1_type == CustomFields::TYPE_DROPDOWN) {
    $left_joins .=  " left join tt_custom_field_log cfl on (l.id = cfl.log_id)".
      " left join tt_custom_field_options cfo on (cfl.option_id = cfo.id)";
  }
}
if ($includeCost && MODE_TIME != $user->tracking_mode)
  $left_joins .= " left join tt_user_project_binds upb on (l.user_id = upb.user_id and l.project_id = upb.project_id)";

// Prepare sql query part for user list.
$user_list_part = null;
if ($user->canManageTeam())
  $user_list_part = " and l.user_id in ($userlist)";
else
  $user_list_part = " and l.user_id = ".$user->id;

// Prepare sql query part for where.
$where = " where l.date >= '".$period->getBeginDate(DB_DATEFORMAT)."' and l.date <= '".$period->getEndDate(DB_DATEFORMAT)."'".
  " $user_list_part $dropdown_parts";
$selected_option = $bean->getAttribute('option');
if ($selected_option)
  $where .= " and l.id in(select log_id from tt_custom_field_log where option_id = $selected_option)";
  
// Obtain regular rows for report.
$rows = array(); // This is our main output for report table.
$filled_dates = array(); // This array represents filled dates in $rows (for dates where we have some data).

$sql = "select ".join(', ', $fields)." from tt_log l $left_joins $where";
$res = $mdb2->query($sql);
if (!is_a($res, 'PEAR_Error')) {
  while ($val = $res->fetchRow()) {
  	  if (isset($val['cost']))
  	    $val['cost'] = str_replace('.', $i18n->getKey('format.float_delimiter'), $val['cost']);
      $row = $val;
      $row['sortkey'] = @$val[$sort_key];
      $filled_dates[] = array('sortkey'=>@$val[$sort_key],'date'=>$val['date']);
      $rows[] = $row;
    }
  } else {
  	die($res->getMessage());
}

// Calculate totals for report summary.
$total_time = 0;
$total_cost = 0;
 
if ($user->canManageTeam() && $includeCost) {
  if (MODE_TIME == $user->tracking_mode) {
  $sql = "select sum(time_to_sec(l.duration)) as time, format(sum(l.billable * coalesce(u.rate, 0) * time_to_sec(l.duration)/3600),2) as cost 
    from tt_log l, tt_users u $where and l.user_id = u.id";
  } else {
    $sql = "select sum(time_to_sec(l.duration)) as time, format(sum(l.billable * coalesce(upb.rate, 0) * time_to_sec(l.duration)/3600),2) as cost 
      from tt_log l
      left join tt_user_project_binds upb on (l.user_id = upb.user_id and l.project_id = upb.project_id) $where";  	
  }
} else
  $sql = "select sum(time_to_sec(l.duration)) as time from tt_log l $where";

$res = $mdb2->query($sql);
if (!is_a($res, 'PEAR_Error')) {
  $val = $res->fetchRow();
  $total_time = sec_to_time_fmt_hm($val['time']);
  if ($includeCost) {
  	$total_cost = $val['cost'];
  	if (!$total_cost) $total_cost = '0.00';
  	$total_cost = str_replace('.', $i18n->getKey('format.float_delimiter'), $total_cost);
  }
} else {
  die($res->getMessage());
}
 
// Determine group by field if we are grouping report data.
switch ($sort_key) {
  case 'user':
    $group_field = 'u.name';
    $group_join = 'left join tt_users u on (l.user_id = u.id) ';
    break;
  case 'project':
  	$group_field = 'p.name';
    $group_join = 'left join tt_projects p on (l.project_id = p.id) ';
  	break;
  case 'task':
  	$group_field = 't.name';
    $group_join = 'left join tt_tasks t on (l.task_id = t.id) ';
  	break;
  case 'cf_1':
  	require_once('../plugins/CustomFields.class.php');
  	$group_field = 'cfo.value';
  	if ($custom_fields->fields[0]['type'] == CustomFields::TYPE_TEXT)
      $group_join = 'left join tt_custom_field_log cfl on (l.id = cfl.log_id) left join tt_custom_field_options cfo on (cfl.value = cfo.id) ';
    else if ($custom_fields->fields[0]['type'] == CustomFields::TYPE_DROPDOWN)
      $group_join = 'left join tt_custom_field_log cfl on (l.id = cfl.log_id) left join tt_custom_field_options cfo on (cfl.option_id = cfo.id) ';
  	break;
  default:
  	$group_field = '';
  	$group_join = '';
}
    
// Calculate subtotals.
$subtotals = array();
if ($group_field) {
  if ($user->canManageTeam()) {
    if (!$includeCost) {
      $sql = "select $group_field as group_field, sum(time_to_sec(l.duration)) as time from tt_log l 
        $group_join $where group by $group_field";
    } else {
      if (MODE_TIME == $user->tracking_mode) {
        $sql = "select $group_field as group_field, sum(time_to_sec(l.duration)) as time, 
          format(sum(l.billable * coalesce(u.rate, 0) * time_to_sec(l.duration)/3600),2) as cost from tt_log l
          left join tt_users u on (u.id = l.user_id)
          $group_join $where group by $group_field";      	
      } else {
      // If we are including cost and tracking projects, our query (the same as above) needs to join the tt_user_project_binds table.
      $sql = "select $group_field as group_field, sum(time_to_sec(l.duration)) as time, 
        format(sum(l.billable * coalesce(upb.rate, 0) * time_to_sec(l.duration)/3600),2) as cost from tt_log l 
        $group_join
        left join tt_user_project_binds upb on (l.user_id = upb.user_id and l.project_id = upb.project_id) $where group by $group_field";
      }
    }
  } else {
  	$sql = "select $group_field as group_field, sum(time_to_sec(l.duration)) as time from tt_log l 
      $group_join $where group by $group_field";
  }
  $res = $mdb2->query($sql);
  if (!is_a($res, 'PEAR_Error')) {
  	while ($val = $res->fetchRow()) {
      if ($includeCost) {
      	$val['cost'] = str_replace('.', $i18n->getKey('format.float_delimiter'), $val['cost']);
      	$subtotals[$val['group_field']] = array('time'=>sec_to_time_fmt_hm($val['time']),'cost'=>$val['cost']);
      } else
        $subtotals[$val['group_field']] = array('time'=>sec_to_time_fmt_hm($val['time']));
    }
  } else {
  	die($res->getMessage());
  }
} else {
	
    	// This needs to be refactored.   
    	if ($totals_only && $bean->getAttribute("project") && count($rows)) {
    		// for show one row in report if was selected "project" and "totals_only"
    		if (is_array($rows[0])) {
    			$pname = $rows[0]["project"];
    			$aname = (isset($rows[0]["activity"])?$rows[0]["activity"]:" ");
    			$subtotals[$pname]['time'] = $total_time;
    			$subtotals[$pname]['cost'] = $total_cost;
    			$sort_key = "project";
    			$rows = array();
				$rows[] = array("sortkey"=>$pname,"user"=>"","date"=>"","project"=>$pname,"activity"=>$aname,"start"=>"","finish"=>"","duration"=>$total_time,"note"=>"", "cost"=>$total_cost,);
    		}
    	} else {
    		$subtotals['common'] = '1';
    	}
    }

// Handling of "Empty days" option. In this case we must show empty records for idle days.
if ($bean->getAttribute('chemptydays')) {
  
  // Fill in subtotals with zeroes where necessary.
	
  // Case of grouping by user.  
  if ($sort_key=='user') {
    $users = ttTeamHelper::getAllUsers($user->team_id);
    $arr = $bean->getAttribute('users');
    foreach ($users as $single_user) {
  	  if (is_array(@$arr) && in_array($single_user['id'], @$arr)) {
        // Fill in zero time and cost subtotals for missing user entries.
  	  	if (!in_array($single_user['name'], array_keys($subtotals))) {
	      $subtotals[$single_user['name']]['time'] = '0:00';
	      $subtotals[$single_user['name']]['cost'] = str_replace('.', $i18n->getKey('format.float_delimiter'), '0.00');
	    }
      }
    }
  }
  
  // Case of grouping by project, project not selected.
  if ($sort_key=='project' && ($bean->getAttribute('project')==null)) {
    $projects = ttTeamHelper::getActiveProjects($user->team_id);
    foreach ($projects as $project) {
      // Fill in zero time and cost subtotals for missing project entries.
      if (!in_array($project['name'], array_keys($subtotals))) {
	    $subtotals[$project['name']]['time'] = '0:00';
	    $subtotals[$project['name']]['cost'] = str_replace('.', $i18n->getKey('format.float_delimiter'), '0.00');
	  }
    }
  }
  
  // Case of grouping by task, task not selected.
  if ($sort_key=='task' && ($bean->getAttribute('task')==null)) {
  	$tasks = ttTeamHelper::getActiveTasks($user->team_id);
  	foreach ($tasks as $task) {
      // Fill in zero time and cost subtotals for missing task entries.
      if (!in_array($task['name'], array_keys($subtotals))) {
        $subtotals[$task['name']]['time'] = '0:00';
        $subtotals[$task['name']]['cost'] = str_replace('.', $i18n->getKey('format.float_delimiter'), '0.00');
      }
  	}
  }
  
  // Add empty rows to our output for idle days.
  foreach ($subtotals as $k=>$v) {
    // Iterate through the entire range of days.
  	foreach ($report_days as $i) {
      $index = $i->toString(DB_DATEFORMAT);
      $row = $empty;
      $row['date'] = $index;
      $row['sortkey'] = ($sort_key=='date' ? $row[$sort_key] : $k);
      $toadd = true;
      // Determine if we need to add an empty row by scanning the $filled_dates array.
      foreach ($filled_dates as $key) {
        if (($key['date'] == $index) && ($key['sortkey'] == $row['sortkey'])) {
          $toadd = false;
        }
      }
      if ($toadd) {
      	// Add an empty row.
        $rows[] = $row;
        // Adjust the $filled_dates array to reflect this.
        $filled_dates[] = array('date'=>$index, 'sortkey'=>$row['sortkey']);
      }
    }
  }
}

// Sort rows so that they are in order.

// Remove colons from start time (needed for sorting).
foreach($rows as $id=>$val) {
  if($val['start']!='')
    $rows[$id]['start'] = str_replace(':', '', $val['start']);
}

// Define sorting string.
$sort_string = "sortkey,date";
if ($bean->getAttribute("chstart")) {
  $sort_string .= ",start";
}
$sort_string .= ",id";

// Sort rows.
$rows = mu_sort($rows, $sort_string);

// Recover colons.
foreach($rows as $id=>$val) {
  if($val['start'] != "") {
  	$len = strlen($val['start']);
  	if($len == 3)
      $rows[$id]['start'] = $val['start']{0}.":".$val['start']{1}.$val['start']{2};
    else
      $rows[$id]['start'] = $val['start']{0}.$val['start']{1}.":".$val['start']{2}.$val['start']{3};
  }
}

//  If we are using 12-hour format we need to convert start and end times to it.
if ('%I:%M %p' == $user->time_format) {
  foreach($rows as $id=>$val) {
    if($val['start'] != '')
      $rows[$id]['start'] = ttTimeHelper::to12HourFormat($val['start']);
    if($val['finish'] != '')
      $rows[$id]['finish'] = ttTimeHelper::to12HourFormat($val['finish']);
  }
}

// TODO: Question: Can sorting be moved inside the idle days block?
// TODO: Refactor the remaining block of code in this file.
// TODO: add a subtotal line on regular report.
// TODO: refactor subtotals['common'] - what is this?
?>